/**************************************************************************
       EquTranslator V4.
***************************************************************************/

#pragma once

#include <tchar.h>
#include <math.h>

//Type definition
#ifdef _USE_CXTRANSLATOR
	typedef double  CmpBaseType;		// Base Type 
	template<class T>  struct TCalc_t{ T real; T imag;};
	typedef TCalc_t<CmpBaseType> TCalc;
	static const TCalc g_zeroTCalc = {0, 0};
	static const TCalc g_hugeTCalc = {HUGE_VAL, HUGE_VAL};
#else
	// Base Type 
	typedef double TCalc;
	static const TCalc g_zeroTCalc = TCalc(0);
	static const TCalc g_hugeTCalc = TCalc(HUGE_VAL);
#endif


/*************************************************************************
User defined function. 
        Function of this type can be added to EquTranslator for further use as 
        part of math expression.
        Arguments:
               _X - a parameter to evaluate in this function (in case of complex 
			   number evaluator the parameter is reference).
        Return: 
               Result of "_X" evaluation.
***************************************************************************/
#ifdef _USE_CXTRANSLATOR
	typedef TCalc (__cdecl *MATHFUNCTION_PTR)(_In_ const TCalc & _X); 
#else
	typedef TCalc (__cdecl *MATHFUNCTION_PTR)(_In_ TCalc _X); 
#endif



/**************************************************************************
Error handle. 
A function of this type can be used as a parameter in InitEquTranslator/InitCxTranslator 
       call to set up callback for error processing.

Arguments:
	errCode - an integer number to define type of error happened in EquTranslator.
	message - a message to describe an error.
	message2 - an additional message (as usual pointing to place in math expression 
				where error happened.
Return: 
	None.
***************************************************************************/
typedef void (__cdecl *TpfErrorHandler)(int errCode, const TCHAR* message, const TCHAR* message2);		



///////////////////////////////////////////////////////////////////////////
//Constants. Error Messages
#define errNoError				 _T("No error.")
#define errMemoryAllocError		 _T("Memory allocation error.")
#define errSyntaxError			 _T("Syntax error: ")
#define errParenthesis			 _T("Syntax error. Please, check parenthesis: ")
#define errSignError			 _T("Syntax error. Sign is used incorectly: ")
#define errNumberError			 _T("Syntax error. Number format is incorect: ")
#define errUnDefFunction		 _T("Uknown function: ")



//Error enum. Should be sync with list of error messages
enum ERR_CODE { eErrNoError					= 0,	//"No error."
				eErrMemoryAllocError		= 1,	//T("Memory allocation error.");
				eErrSyntaxError				= 2,	//_T("Syntax error: ");
				eErrParenthesis				= 3,	//_T("Syntax error. Please, check parenthesis: ");
				eErrSignError				= 4,	//_T("Syntax error. Sign is used incorectly: ");
				eErrNumberError				= 5,	//_T("Syntax error. Number format is incorect: ");
				eErrUnDefFunction			= 6,	//_T("Uknown function: ");
				eEndElement
				} ;



typedef struct {ERR_CODE code;  const TCHAR *message;} ERR_MESSAGE;


const ERR_MESSAGE g_errMessage[eEndElement] = {
	{eErrNoError,					errNoError},
	{eErrMemoryAllocError,			errMemoryAllocError},
	{eErrSyntaxError,				errSyntaxError},		
	{eErrParenthesis,				errParenthesis},
	{eErrSignError,					errSignError},
	{eErrNumberError,				errNumberError},		
	{eErrUnDefFunction,				errUnDefFunction}
};